
//		Functions to do math on the word at the cursor.


#ifndef MATH_WORD_VER
#define MATH_WORD_VER			1.2.0.1



/*
Examines a command line, gets the space-delimited 'word' at the cursor, and
pushes it through Evaluate().  String is the command line; *Cursor is the offset
of the cursor within the string (0 = the first character), and Redraw controls
whether ClearEditLine() is called.

Returns:	-3  canceled by user
			-2  other error (buffer overflow)
			-1  bad parameter (null pointer)
			 0  no expression found, string not changed
			 1  expression found and expanded
*/


int DoMath( LPTSTR String, int *Cursor, BOOL Redraw )
{
	if ( !String || !Cursor )
		return -1;
	else if ( String[0] == '\0' )
		return 0;


	wchar_t ch		= '\0';
	int WordStart	= -1;
	int  WordEnd	= -1;
	BOOL quote		= FALSE;
	BOOL inWord		= FALSE;


	for ( int i = 0; String[i]; i++ ) {
		ch = String[i];

		if ( ch == QUOTE )
			quote = !quote;

		if ( !iswspace( ch ) && ( i <= *Cursor ) ) {
			if ( !inWord )
				WordStart = i;

			inWord = TRUE;
		}

		if ( !iswspace( ch ) && inWord )
			WordEnd = i;

		if ( iswspace( ch ) && !quote )
			inWord = FALSE;
	}

	if ( ( WordStart < 0 ) || ( WordEnd < ( *Cursor - 1 ) ) )
		return 0;


	wchar_t VarBuffer[MAX_CMD_LINE_LEN]		= L"";
	wchar_t NewLine[MAX_CMD_LINE_LEN]		= L"";

	int len		= WordEnd - WordStart + 1;
	int j		= 0;


	if ( ( len + 8 ) >= ( MAX_CMD_LINE_LEN ) )
		return -2;

	wcsncpy_s( VarBuffer, MAX_CMD_LINE_LEN, String + WordStart, len );
	TrimQuotes( VarBuffer );

	if ( VarBuffer[0] == '\0' )
		return 0;


	HANDLE OldStderrHandle	= GetStdHandle( STD_ERROR_HANDLE );
	HANDLE OldStdoutHandle	= GetStdHandle( STD_OUTPUT_HANDLE );
	HANDLE NewStderrHandle	= CreateFile( L"nul", GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_ALWAYS, 0, NULL );
	int rv = 0;

	if ( NewStderrHandle != INVALID_HANDLE_VALUE ) {
		SetStdHandle( STD_ERROR_HANDLE, NewStderrHandle );		//  redirect stderr to nul:
		SetStdHandle( STD_OUTPUT_HANDLE, NewStderrHandle );		//  redirect stdout to nul:
	}

	rv = Evaluate( VarBuffer );									//	and evaluate the expression

	if ( NewStderrHandle != INVALID_HANDLE_VALUE ) {
		SetStdHandle( STD_ERROR_HANDLE, OldStderrHandle );		//  put stderr back where we found it
		SetStdHandle( STD_OUTPUT_HANDLE, OldStdoutHandle );		//  put stdout back where we found it
		CloseHandle( NewStderrHandle );
	}

	if ( rv )												//  if any error expanding the expression,
		return 0;											//  exit with no change

	if ( Redraw )
		ClearEditLine( String );							//  clear the command line if desired


	for ( int i = 0; i < WordStart; i++ )					//  copy everything from before the expression to NewLine
		if ( j < ( MAX_CMD_LINE_LEN - 1 ) )
			NewLine[j++] = String[i];

	for ( int i = 0; VarBuffer[i]; i++ )					//  then append the value of the expression
		if ( j < ( MAX_CMD_LINE_LEN - 1 ) )
			NewLine[j++] = VarBuffer[i];

	for ( int i = WordEnd + 1; String[i]; i++ )				//  and then everything after the expression
		if ( j < ( MAX_CMD_LINE_LEN - 1 ) )
			NewLine[j++] = String[i];

	NewLine[j] = '\0';										//  null-terminate the string

	wcscpy_s( String, MAX_CMD_LINE_LEN, NewLine );			//  copy the updated string over the original
	*Cursor = WordStart + (int) wcslen( VarBuffer );		//  and update the cursor position
	return 1;												//  exit with changes
}



int MathWord( LPKEYINFO24 KeyInfo )
{
	int CursorPosition		= (int) ( KeyInfo->pszCurrent - KeyInfo->pszLine );
	int rv					= DoMath( KeyInfo->pszLine, &CursorPosition, TRUE );


	if ( rv == 1 ) {
		KeyInfo->pszCurrent		= KeyInfo->pszLine + CursorPosition;
		KeyInfo->fRedraw		= 1;
	}
	else if ( rv != -3 )
		ErrorBeep();

	KeyInfo->nKey = 0;
	if ( NewKeyHook )
		KeyInfo->pszKey[0] = '\0';

	return 0;
}


#endif	//	MATH_WORD_VER



/*

	1.0.0.0		2022-05-03

	1.0.1.0		2022-05-05
				Calls TrimQuotes() to remove double quotes around the
				expression. 

	1.1.0.0		2022-05-06
				Added ExpandWord() and its dependencies.

	1.2.0.0		2022-05-12
				Ripped it all out again.  Moved all the variable expansion
				stuff out to ExpandExpression.cpp.

	1.2.0.1		2022-12-20
				Spelling fix; no changes to functionality.

*/
