
//		PrintViewport.cpp

//		Dumps the currently visible area of the console buffer to a .PNG file,
//		then prints the file using the default handler.


#ifndef PRINT_VIEWPORT_VER
#define PRINT_VIEWPORT_VER					1.0.5.2


#define PRINT_VIEWPORT_FLAG_AUTO_OPEN		0x0400

#define PRINT_VIEWPORT_EXT_LEN				16


#include <ShlObj.h>
#include "FindMyWindow.cpp"



const wchar_t PVP_SavePicDirVarName[]		= L"SavePicDir";
const wchar_t PVP_SavePicExtVarName[]		= L"SavePicExt";
const wchar_t PVP_SavePicFlagsVarName[]		= L"SavePicFlags";

const wchar_t PVP_DefaultExt[]				= L".png";


//	--------------------------------------------------------------------
//	Constants for graphics file types, file extensions, and mime types:

#define PVP_NUM_GRAF_EXTENSIONS		7
#define PVP_NUM_GRAF_TYPES			5
#define PVP_GRAF_TYPE_BMP			1
#define PVP_GRAF_TYPE_JPEG			2
#define PVP_GRAF_TYPE_PNG			3
#define PVP_GRAF_TYPE_GIF			4
#define PVP_GRAF_TYPE_TIFF			5

const wchar_t PVP_GrafExtension[PVP_NUM_GRAF_EXTENSIONS][6]		= { L".bmp", L".jpg", L".jpeg", L".png", L".gif", L".tif", L".tiff" };

unsigned int PVP_GrafType[PVP_NUM_GRAF_EXTENSIONS]				= { PVP_GRAF_TYPE_BMP, PVP_GRAF_TYPE_JPEG, PVP_GRAF_TYPE_JPEG, PVP_GRAF_TYPE_PNG, PVP_GRAF_TYPE_GIF,
																	PVP_GRAF_TYPE_TIFF, PVP_GRAF_TYPE_TIFF };

const wchar_t PVP_GrafMimeType[PVP_NUM_GRAF_TYPES][11]			= { L"image/bmp", L"image/jpeg", L"image/png", L"image/gif", L"image/tiff" };

//	--------------------------------------------------------------------



int PVP_SavePicFilename( LPTSTR Filename, size_t buflen, LPCTSTR Ext )
{
	if ( !Filename || ( buflen < MAX_PATH ) )
		return -1;

	if ( !Ext )
		Ext = PVP_SavePicExtVarName;


	wchar_t Path[SHORT_BUF_LEN]			= L"";
	wchar_t Name[SHORT_BUF_LEN]			= L"";
	wchar_t FullName[MAX_FILENAME_LEN]	= L"";

	SYSTEMTIME Now;


	Filename[0]	= '\0';
	Path[0]		= '\0';
	GetEnvironmentVariable( PVP_SavePicDirVarName, Path, SHORT_BUF_LEN );

	if ( Path[0] ) {
		if ( wcschr( Path, '%' ) )
			ExpandVariablesQuietly( Path );

		if ( !QueryIsFileOrDirectory( Path ) )
			CreateDirectory( Path, NULL );

		if ( !QueryIsDirectory( Path ) )
			Path[0] = '\0';
	}

	if ( Path[0] == '\0' ) {
		SHGetFolderPath( NULL, CSIDL_DESKTOPDIRECTORY, NULL, SHGFP_TYPE_CURRENT, Path );

		if ( Path[0] )
			if ( !QueryIsDirectory( Path ) )
				Path[0] = '\0';
	}

	if ( Path[0] == '\0' )
		return 0;


	GetLocalTime( &Now );

	wsprintf( Name, L"Pic %04u-%02u-%02u %02u.%02u.%02u%s", Now.wYear, Now.wMonth, Now.wDay, Now.wHour, Now.wMinute, Now.wSecond, Ext );
	wcscpy_s( FullName, MAX_FILENAME_LEN, Path );
	AppendFilename( FullName, Name, MAX_FILENAME_LEN );

	if ( QueryIsFileOrDirectory( FullName ) ) {
		for ( unsigned int i = 1; i < 1001; i++ ) {

			wsprintf( Name, L"Pic %04u-%02u-%02u %02u.%02u.%02u (%u)%s", Now.wYear, Now.wMonth, Now.wDay, Now.wHour, Now.wMinute, Now.wSecond, i, Ext );
			wcscpy_s( FullName, MAX_FILENAME_LEN, Path );
			AppendFilename( FullName, Name, MAX_FILENAME_LEN );
			if ( !QueryIsFileOrDirectory( FullName ) )
				break;
		}

		if ( QueryIsFileOrDirectory( FullName ) )
			FullName[0] = '\0';
	}

	if ( FullName[0] == '\0' )
		return 0;

	if ( wcslen( FullName ) >= buflen )
		return 0;

	wcscpy_s( Filename, buflen, FullName );
	return 1;
}



int PVP_IsGraphicsExtension( LPCTSTR Ext )
{
	if ( !Ext )
		return 0;

	for ( unsigned int i = 0; i < PVP_NUM_GRAF_EXTENSIONS; i++ )
		if ( !_wcsicmp( Ext, PVP_GrafExtension[i] ) )
			return PVP_GrafType[i];

	return 0;
}



int PVP_GetEncoderClsid( LPCTSTR Format, CLSID *Clsid )
{
	unsigned int num	= 0;          // number of image encoders
	unsigned int size	= 0;         // size of the image encoder array in bytes

	ImageCodecInfo *CodecInfo	= NULL;


	GetImageEncodersSize( &num, &size );
   
	if ( !size )
		return -1;  // Failure


	CodecInfo = (ImageCodecInfo*) malloc( size );
	if ( !CodecInfo )
		return -1;  // Failure

	GetImageEncoders( num, size, CodecInfo );

	for ( unsigned int j = 0; j < num; ++j ) {
		if ( !wcscmp( CodecInfo[j].MimeType, Format ) ) {
			*Clsid = CodecInfo[j].Clsid;
			free( CodecInfo );
			return j;  // Success
		}    
	}

	free( CodecInfo );
	return -1;  // Failure
}



int PVP_GetEncoderClsidByExtension( LPTSTR Filename, CLSID *Clsid )
{
	if ( !Filename || !Clsid )
		return 0;


	int rv		= PVP_IsGraphicsExtension( _ExtensionPart( Filename, NULL ) );

	if ( rv )
		PVP_GetEncoderClsid( PVP_GrafMimeType[rv - 1], Clsid );

	return rv;
}



int PVP_SaveWindowPic( HDC WindowDC, LPTSTR Filename, BOOL Overwrite, int Left, int Top, int Width, int Height )
{
	if ( !WindowDC || !Filename )
		return -1;
	else if ( Filename[0] == '\0' )
		return -1;


	CLSID EncoderClsid;
	HDC CaptureDC				= NULL;
	int ColorDepth				= 0;
	HBITMAP CaptureBM			= NULL;
	HGDIOBJ OldObject			= NULL;

	Bitmap *Snapshot			= NULL;
	EncoderParameters EncoderParams;
	ULONG Quality				= 100;

	int rv = 1;



	if ( !PVP_GetEncoderClsidByExtension( Filename, &EncoderClsid ) )
		return UnknownImageFormat;

	if ( IsAFile( Filename ) && !Overwrite )
		return ERROR_FILE_EXISTS | 0x8000;


	EncoderParams.Count		= 1;
	EncoderParams.Parameter[0].NumberOfValues	= 1;
	EncoderParams.Parameter[0].Guid				= EncoderQuality;
	EncoderParams.Parameter[0].Type				= EncoderParameterValueTypeLong;
	EncoderParams.Parameter[0].Value			= &Quality;


	CaptureDC = CreateCompatibleDC( WindowDC );

	if ( CaptureDC ) {
		CaptureBM = CreateCompatibleBitmap( WindowDC, Width, Height );

		if ( CaptureBM ) {
			OldObject = SelectObject( CaptureDC, CaptureBM );

			if ( OldObject && ( OldObject != HGDI_ERROR ) ) {

				if ( BitBlt( CaptureDC, 0, 0, Width, Height, WindowDC, Left, Top, SRCCOPY ) ) {

					Snapshot = new Bitmap( CaptureBM, (HPALETTE) NULL );
					if ( Snapshot ) {
						rv = Snapshot->Save( Filename, &EncoderClsid, &EncoderParams );
						delete Snapshot;
					}
				}

				SelectObject( CaptureDC, OldObject );
			}

			DeleteObject( CaptureBM );
		}

		DeleteDC( CaptureDC );
	}

	return rv;
}



int PrintViewport( KEYINFO24 *ki, BOOL SavePic )
{
	wchar_t Filename[MAX_FILENAME_LEN]		= L"";
	wchar_t Ext[PRINT_VIEWPORT_EXT_LEN]		= L"";

	HWND ViewWindow				= NULL;
	unsigned int WindowType		= 0;
	HDC WindowDC				= NULL;
	WINDOWINFO WindowInfo;

	int Left					= 0;
	int Top						= 0;
	int Width					= 0;
	int Height					= 0;

	unsigned int Flags			= EnvVarValue( PVP_SavePicFlagsVarName, 0 );
	int rv						= 0;
	BOOL Success				= FALSE;


	ki->nKey = 0;
	if ( NewKeyHook )
		ki->pszKey[0] = '\0';


	GetEnvironmentVariable( PVP_SavePicExtVarName, Ext, PRINT_VIEWPORT_EXT_LEN );
	if ( !PVP_IsGraphicsExtension( Ext ) )
		wcscpy_s( Ext, PRINT_VIEWPORT_EXT_LEN, PVP_DefaultExt );

	if ( SavePic )
		PVP_SavePicFilename( Filename, MAX_FILENAME_LEN, Ext );
	else
		CreateTempFilename( Filename, MAX_FILENAME_LEN, Ext, NULL );

	if ( Filename[0] == '\0' )
		goto CleanupAndExit;


	FindMyWindow( &WindowType, &ViewWindow );
	//	FmtMsgBox( 0, PLUGIN_NAME, L"ViewWindow = 0x%08X\nWindowType = %u", ViewWindow, WindowType );


	if ( ViewWindow ) {
		memset( &WindowInfo, 0x00, sizeof( WINDOWINFO ) );
		WindowInfo.cbSize = sizeof( WINDOWINFO );
		GetWindowInfo( ViewWindow, &WindowInfo );

		if ( WindowType >= MY_WINDOW_WINDOWS_TERMINAL ) {
			Left	= WindowInfo.rcClient.left;
			Top		= WindowInfo.rcClient.top;
			Width	= WindowInfo.rcClient.right  - WindowInfo.rcClient.left + 1;
			Height	= WindowInfo.rcClient.bottom - WindowInfo.rcClient.top  + 1;

			WindowDC = GetDC( NULL );
		}
		else {
			Left	= 0;
			Top		= 0;
			Width	= WindowInfo.rcClient.right  - WindowInfo.rcClient.left + 1;
			Height	= WindowInfo.rcClient.bottom - WindowInfo.rcClient.top  + 1;

			WindowDC = GetDC( ViewWindow );
		}

		if ( WindowDC ) {
			rv = PVP_SaveWindowPic( WindowDC, Filename, FALSE, Left, Top, Width, Height );

			ReleaseDC( ( WindowType >= MY_WINDOW_WINDOWS_TERMINAL ) ? NULL : ViewWindow, WindowDC );

			if ( !rv ) {
				if ( SavePic ) {
					Success = TRUE;

					if ( Flags & PRINT_VIEWPORT_FLAG_AUTO_OPEN )
						ShellExecute( NULL, NULL, Filename, NULL, NULL, SW_SHOW );
				}
				else if ( (INT_PTR) ShellExecute( NULL, L"Print", Filename, NULL, NULL, SW_HIDE ) > 32 )
					Success = TRUE;
			}
		}
	}


CleanupAndExit:

	if ( !Success )
		ErrorBeep();

	return 0;
}


#endif	//	PRINT_VIEWPORT_VER


/*

	1.0.0.0		2022-07-11

	1.0.1.0		2022-07-13
				Added an option to save the picture to the desktop, instead of
				saving it to the temp directory and printing it.  Added support
				for Console2, Windows Terminal.

	1.0.2.0		2022-07-14
				Added support for TCC/LE in a Take Command tab.  Use ShellExecute()
				instead of LaunchFile().

	1.0.3.0		2022-07-14
				Added support for image types other than .PNG.

	1.0.4.0		2022-07-15
				Added support for ConEmu.

	1.0.5.0		2022-08-11
				Accommodate changes in FindMyWindow.cpp.

	1.0.5.1		2022-08-29
				Remove debugging message box.

	1.0.5.2		2023-06-13
				Removed a call to QueryIsFile() to work around an issue in
				TCC v30.00.20.

*/
